@@ -27,7 +27,7 @@ __isl_give isl_basic_set *isl_basic_set_from_vec(__isl_take isl_vec *vec);
 
 int isl_tab_set_initial_basis_with_cone(struct isl_tab *tab,
 	struct isl_tab *tab_cone);
-struct isl_vec *isl_tab_sample(struct isl_tab *tab);
+__isl_give isl_vec *isl_tab_sample(struct isl_tab *tab);
 
 #if defined(__cplusplus)
 }
